## $Id: tests.py 8521 2012-05-25 10:33:48Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import os
import shutil
import tempfile
import datetime
from zope.intid.interfaces import IIntIds
from zope.interface.verify import verifyClass, verifyObject
from zope.component.hooks import setSite, clearSite
from zope.component import createObject, getUtility
from zope.catalog.interfaces import ICatalog
from zope.testbrowser.testing import Browser
from waeup.kofa.app import University
from waeup.kofa.university.faculty import Faculty
from waeup.kofa.university.department import Department
from waeup.kofa.testing import FunctionalTestCase
from waeup.kofa.applicants.container import ApplicantsContainer
from waeup.kofa.applicants.tests.test_batching import ApplicantImportExportSetup
from waeup.kofa.interfaces import IBatchProcessor
from waeup.aaue.testing import FunctionalLayer
from waeup.aaue.applicants.export import CustomApplicantsExporter


class ApplicantUITest(FunctionalTestCase):
    """Perform some browser tests.
    """
    layer = FunctionalLayer

    def setUp(self):
        super(ApplicantUITest, self).setUp()
        # Setup a sample site for each test
        app = University()
        self.dc_root = tempfile.mkdtemp()
        app['datacenter'].setStoragePath(self.dc_root)

        # Prepopulate the ZODB...
        self.getRootFolder()['app'] = app
        # we add the site immediately after creation to the
        # ZODB. Catalogs and other local utilities are not setup
        # before that step.
        self.app = self.getRootFolder()['app']
        # Set site here. Some of the following setup code might need
        # to access grok.getSite() and should get our new app then
        setSite(app)

        # Add an two different applicants containers
        self.pgcontainer = ApplicantsContainer()
        self.pgcontainer.code = u'pgft2011'
        self.pgcontainer.prefix = u'pgft'
        self.app['applicants']['pgft2011'] = self.pgcontainer
        self.ugcontainer = ApplicantsContainer()
        self.ugcontainer.code = u'app2011'
        self.ugcontainer.prefix = u'app'
        self.app['applicants']['app2011'] = self.ugcontainer

        # Populate university
        self.certificate = createObject('waeup.Certificate')
        self.certificate.code = 'CERT1'
        self.certificate.application_category = 'basic'
        self.certificate.start_level = 100
        self.certificate.end_level = 500
        self.app['faculties']['fac1'] = Faculty()
        self.app['faculties']['fac1']['dep1'] = Department()
        self.app['faculties']['fac1']['dep1'].certificates.addCertificate(
            self.certificate)

        # Add (customized) applicants
        pgapplicant = createObject(u'waeup.Applicant')
        pgapplicant.firstname = u'Anna'
        pgapplicant.lastname = u'Post'
        self.app['applicants']['pgft2011'].addApplicant(pgapplicant)
        self.pgapplication_number = pgapplicant.application_number
        self.pgapplicant = self.app['applicants']['pgft2011'][
            self.pgapplication_number]

        ugapplicant = createObject(u'waeup.Applicant')
        ugapplicant.firstname = u'Klaus'
        ugapplicant.lastname = u'Under'
        self.app['applicants']['app2011'].addApplicant(ugapplicant)
        self.ugapplication_number = ugapplicant.application_number
        self.ugapplicant = self.app['applicants']['app2011'][
            self.ugapplication_number]

        self.browser = Browser()
        self.browser.handleErrors = False

    def tearDown(self):
        super(ApplicantUITest, self).tearDown()
        shutil.rmtree(self.dc_root)
        clearSite()
        return

    def test_manage_and_view_applicant(self):
        # Managers can manage pg applicants
        pgapplicant_path = ('http://localhost/app/applicants/pgft2011/%s'
            % self.pgapplication_number)
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        # The IPGApplicant interface is really used in all pages
        self.browser.open(pgapplicant_path)
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertTrue('Employer' in self.browser.contents)
        self.browser.open(pgapplicant_path + '/manage')
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertTrue('Employer' in self.browser.contents)
        self.browser.open(pgapplicant_path + '/edit')
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertTrue('Employer' in self.browser.contents)
        self.browser.open(pgapplicant_path + '/application_slip.pdf')
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        # If we view the applicant in the ug container,
        # the employer field doesn't appear
        ugapplicant_path = ('http://localhost/app/applicants/app2011/%s'
            % self.ugapplication_number)
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(ugapplicant_path)
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertFalse('Employer' in self.browser.contents)
        self.browser.open(ugapplicant_path + '/manage')
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        # We can save the applicant
        self.browser.getControl(name="form.reg_number").value = '1234'
        self.browser.getControl(name="form.firstname").value = 'John'
        self.browser.getControl(name="form.lastname").value = 'Tester'
        self.browser.getControl(name="form.course1").value = ['CERT1']
        self.browser.getControl(name="form.date_of_birth").value = '09/09/1988'
        self.browser.getControl(name="form.lga").value = ['foreigner']
        self.browser.getControl(name="form.sex").value = ['m']
        self.browser.getControl(name="form.email").value = 'xx@yy.zz'
        self.browser.getControl("Save").click()
        self.assertMatches('...Form has been saved...', self.browser.contents)
        self.assertFalse('Employer' in self.browser.contents)
        self.browser.open(ugapplicant_path + '/edit')
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertFalse('Employer' in self.browser.contents)
        self.browser.open(ugapplicant_path + '/application_slip.pdf')
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        return

class ApplicantsExporterTest(ApplicantImportExportSetup):

    layer = FunctionalLayer

    def setUp(self):
        super(ApplicantsExporterTest, self).setUp()
        self.outfile = os.path.join(self.workdir, 'myoutput.csv')
        self.cat = getUtility(ICatalog, name='applicants_catalog')
        self.intids = getUtility(IIntIds)
        return

    def setup_applicant(self, applicant):
        # set predictable values for `applicant`
        applicant.reg_number = u'123456'
        applicant.applicant_id = u'dp2011_654321'
        applicant.firstname = u'Anna'
        applicant.lastname = u'Tester'
        applicant.middlename = u'M.'
        applicant.date_of_birth = datetime.date(1981, 2, 4)
        applicant.sex = 'f'
        applicant.email = 'anna@sample.com'
        applicant.phone = u'+234-123-12345'
        applicant.course1 = self.certificate
        applicant.course2 = self.certificate
        applicant.course_admitted = self.certificate
        applicant.notice = u'Some notice\nin lines.'
        applicant.screening_score = 98
        applicant.screening_venue = u'Exam Room'
        applicant.password = 'any password'
        return applicant

    def test_export_all(self):
        # we can export all applicants in a portal
        # set values we can expect in export file
        self.applicant = self.setup_applicant(self.applicant)
        exporter = CustomApplicantsExporter()
        exporter.export_all(self.app, self.outfile)
        result = open(self.outfile, 'rb').read()
        # The exported records do contain a real date in their
        # history dict. We skip the date and split the comparison
        # into two parts.
        self.assertTrue(
            'applicant_id,application_date,application_number,course1,course2,'
            'course_admitted,date_of_birth,display_fullname,email,emp2_end,'
            'emp2_position,emp2_reason,emp2_start,emp_end,emp_position,'
            'emp_reason,emp_start,employer,employer2,firstname,history,'
            'hq_degree,hq_disc,hq_matric_no,hq_school,hq_session,hq_type,'
            'lastname,lga,locked,middlename,nationality,notice,nysc_lga,'
            'nysc_year,password,perm_address,phone,pp_school,presently_inst,'
            'reg_number,screening_score,screening_venue,sex,state,student_id'
            in result)
        self.assertTrue(
            'Application initialized by system\'],,,,,,,Tester,,0,M.,,'
            '"Some notice\nin lines.",,,any password,,+234-123-12345,,,'
            '123456,98,Exam Room,f,initialized,' in result)
        return